import { useRef, useState } from 'react';
import { useAuthState } from 'react-firebase-hooks/auth';
import './FileUpload.scss';

import { Button } from '@chakra-ui/react';
import { Auth } from 'firebase/auth';
import { FirebaseStorage, ref, uploadBytes } from 'firebase/storage';
import localUpload from './uploadProviders/localUpload';

/**
 * A componenet for uploading files to Firebase Storage
 * Can be used with different upload source providers (local, gdrive, onedrive)
 * @param firebase Contains instances of Firebase services.
 * @param firebase.storage FirebaseStorage instance. 
 * @param firebase.auth Firbase Auth instance.
 * @returns 
 */
export default function FileUpload(firebase: {storage: FirebaseStorage, auth: Auth}) {
  const fileInputRef = useRef<HTMLInputElement>(null);
  const [selectedFile, setSelectedFile] = useState<File | null>();
  const [fileName, setFileName] = useState<string | null>();
  const [uploadIsDisabled, setUploadIsDisabled] = useState(true);
  const [uploadInProgress, setUploadInProgress] = useState(false);
  const [buttonText, setButtonText] = useState('Select your file first');

  
  const [user] = useAuthState(firebase.auth);

  // choose different upload provider (local, google, ms, etc..)
  const onAreaClick = (e: React.MouseEvent<HTMLDivElement>) => {
    e.preventDefault();
    localUpload(fileInputRef)
  }


  // Handling file selection from input
  const onFileSelected = (e: React.ChangeEvent<HTMLInputElement>) => {
    if (e.target.files && e.target.files[0]) {
      setSelectedFile(e.target.files[0]);
      setFileName(e.target.files[0].name);
      setUploadIsDisabled(false); // Enabling upload button
      setButtonText("Upload file");
    }
  };


  const handleFileUpload = async (e: React.FormEvent<HTMLFormElement>) => {
    e.preventDefault();
    setUploadInProgress(true);
    setUploadIsDisabled(true);
    setButtonText("Wait we're uploading your file...");

    try {
      if (selectedFile && user?.uid && fileName) {
        // assemble firebase storage path
        const userDirRef = ref(firebase.storage, user.uid);
        const fileRef = ref(userDirRef, fileName);
        // return snapshot
        await uploadBytes(fileRef, selectedFile);
        

        // Reset to default values after 3 seconds
        setTimeout(() => {
          setUploadInProgress(false);
          setSelectedFile(null);
          setFileName(null);
          setButtonText('Select your file first');
        }, 3000);
      }
    } catch (error) {
      setFileName(null);
      setUploadInProgress(false);

      setTimeout(() => {
        setButtonText('Select your file first');
      }, 1500);
    }
  };


  return (
    <div className='fileUpload'>
      <h1>Upload a file to Firebase Cloud Storage</h1>
      <form onSubmit={handleFileUpload}>
        <label className='uploader'>
          <div className='upload-space' onClick={onAreaClick}>
          </div>
          <p className='filename'>
            {fileName ? fileName : 'No file selected yet'}
          </p>
          <input ref={fileInputRef} disabled={uploadInProgress} className='fileInput' type='file' onChange={onFileSelected}></input>
        </label>
        <Button
          type='submit'
          colorScheme='teal'
          className='btn'
          variant='solid'
          isDisabled={uploadIsDisabled}
          isLoading={uploadInProgress}
          tabIndex={0}
        >{buttonText}</Button>
      </form>
    </div>
  )
}